/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIPrefBranch.idl"

/**
 * An empty interface to provide backwards compatibility for existing code.
 *
 * @see nsIPrefBranch
 */
[scriptable, uuid(8892016d-07f7-4530-b5c1-d73dfcde4a1c)]
interface nsIPrefBranch2 : nsIPrefBranch
{
};
