/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMCSSConditionRule.idl"

/**
 * Interface for @-moz-document rules in the CSS OM.
 */
[scriptable, uuid(f118a5a8-ac36-464f-b993-18cf6fe76fda)]
interface nsIDOMCSSMozDocumentRule : nsIDOMCSSConditionRule
{
  // XXX Add access to the URL list.
};
